﻿using System;
using System.Data.Entity;
using BL.AppInfrastructure;
using BL.Repositories.UserAccount;
using BL.Services;
using BL.Utils.PriceCalculators;
using BL.Utils.Resolvers;
using BrockAllen.MembershipReboot;
using Castle.MicroKernel.Registration;
using Castle.MicroKernel.Resolvers.SpecializedResolvers;
using Castle.MicroKernel.SubSystems.Configuration;
using Castle.Windsor;
using DAL;
using Riganti.Utils.Infrastructure.Core;
using Riganti.Utils.Infrastructure.EntityFramework;
using UserAccount = Riganti.Utils.Infrastructure.EntityFramework.UserAccount;

namespace BL.Bootstrap
{
    public class BussinessLayerInstaller : IWindsorInstaller
    {
        public void Install(IWindsorContainer container, IConfigurationStore store)
        {
            container.Register(

                Component.For<Func<DbContext>>()
                    .Instance(() => new DemoEshopDbContext())
                    .LifestyleTransient(),

                Component.For<IUnitOfWorkProvider>()
                    .ImplementedBy<AppUnitOfWorkProvider>()
                    .LifestyleSingleton(),

                Component.For<IUnitOfWorkRegistry>()
                    .Instance(new HttpContextUnitOfWorkRegistry(new ThreadLocalUnitOfWorkRegistry()))
                    .LifestyleSingleton(),
                
                Component.For<IUserAccountRepository<UserAccount>>()
                    .ImplementedBy<UserAccountManager>()
                    .LifestyleTransient(),
                
                Component.For(typeof(IRepository<,>))
                    .ImplementedBy(typeof(EntityFrameworkRepository<,>))
                    .LifestyleTransient(),
                
                Component.For<IPriceCalculatorResolver>()
                    .ImplementedBy<PriceCalculatorResolver>()
                    .LifestyleSingleton(),

                Classes.FromAssemblyContaining<AppUnitOfWork>()
                    .BasedOn(typeof (AppQuery<>))
                    .LifestyleTransient(),

                Classes.FromThisAssembly()
                    .BasedOn<IPriceCalculator>()
                    .WithService.FromInterface()
                    .LifestyleSingleton(),
                
                Classes.FromAssemblyContaining<AppUnitOfWork>()
                    .BasedOn(typeof(IRepository<,>))
                    .LifestyleTransient(),

                Classes.FromThisAssembly()
                    .BasedOn<EshopService>()                   
                    .WithServiceDefaultInterfaces()                   
                    .LifestyleTransient(),

                Classes.FromThisAssembly()
                    .InNamespace("BL.Facades")
                    .LifestyleTransient()
            );

            // add collection subresolver in order to resolve IEnumerable in Price Calculator Resolver
            container.Kernel.Resolver.AddSubResolver(new CollectionResolver(container.Kernel));
        }
    }
}
