﻿using AutoMapper;
using BL.DTOs.Categories;
using BL.DTOs.Coupons;
using BL.DTOs.Customers;
using BL.DTOs.Orders;
using BL.DTOs.Products;
using BL.DTOs.Reviews;
using BL.DTOs.Stocks;
using DAL.Entities;
using Riganti.Utils.Infrastructure.EntityFramework;

namespace BL.Bootstrap
{
    public static class MappingInit
    {
        public static void ConfigureMapping()
        {
            Mapper.Initialize(config =>
            {
                config.CreateMap<Category, CategoryDTO>().ReverseMap();

                config.CreateMap<Customer, CustomerDTO>()
                    .ForMember(dest => dest.Email, opts => opts.MapFrom(src => src.Account.Email))
                    .ForMember(dest => dest.FirstName, opts => opts.MapFrom(src => src.Account.FirstName))
                    .ForMember(dest => dest.LastName, opts => opts.MapFrom(src => src.Account.LastName))
                    .ForMember(dest => dest.MobilePhoneNumber, opts => opts.MapFrom(src => src.Account.MobilePhoneNumber))
                    .ForMember(dest => dest.Address, opts => opts.MapFrom(src => src.Account.Address))
                    .ForMember(dest => dest.BirthDate, opts => opts.MapFrom(src => src.Account.BirthDate))
                    .ReverseMap();

                
                config.CreateMap<Coupon, CouponDTO>()
                    .ForMember(dest => dest.CustomerID, option => option.MapFrom(coupon => coupon.Customer.ID))
                    .ForMember(dest => dest.OrderID, option => option.MapFrom(coupon => coupon.Order != null ? coupon.Order.ID : 0))
                    .ReverseMap();

                config.CreateMap<Order, OrderDTO>()
                    .ForMember(dest => dest.CustomerID, option => option.MapFrom(order => order.Customer.ID))
                    .ReverseMap();

                config.CreateMap<OrderItem, OrderItemDTO>().ReverseMap();

                config.CreateMap<Product, ProductDTO>()
                    .ForMember(productDto => productDto.CategoryID, option => option.MapFrom(product => product.Category.ID))
                    .ForMember(productDto => productDto.StockID, option => option.MapFrom(product => product.Stock.ID))
                    .ReverseMap();

                config.CreateMap<ProductReview, ProductReviewDTO>()
                    .ForMember(dest => dest.ProductID, opts => opts.MapFrom(src => src.Product.ID));

                config.CreateMap<ProductReviewEditDTO, ProductReview>();

                config.CreateMap<ProductReviewCreateDTO, ProductReview>();

                config.CreateMap<Stock, StockDTO>()
                    .ForMember(stockDto => stockDto.ProductID, option => option.MapFrom(stock => stock.Product.ID))
                    .ReverseMap();
            });
        }
    }
}
