﻿using System;
using System.ComponentModel.DataAnnotations;

namespace BL.DTOs.Coupons
{
    /// <summary>
    /// Coupon with promo code discount for the overall order
    /// </summary>
    public class CouponDTO
    {
        public int ID { get; set; }

        [Range(0, 100)]
        public int DiscountPercentage { get; set; }

        [MaxLength(8)]
        [Required]
        public string PromoCode { get; set; }

        public bool IsUsed => this.OrderID.HasValue;

        [Required]
        [DataType(DataType.DateTime)]
        [DisplayFormat(DataFormatString = "{0:d/M/yyyy HH:mm:ss}")]
        public DateTime Issued { get; set; }

        public int CustomerID { get; set; }

        public int? OrderID { get; set; }

        public override string ToString()
        {
            return $"Promo code: {PromoCode} for customer with ID: {CustomerID} with {DiscountPercentage}% discount.";
        }
    }
}
