﻿using System.ComponentModel.DataAnnotations;
using BL.DTOs.Products;

namespace BL.DTOs.Orders
{
    /// <summary>
    /// Wrapper class for single item within customer order
    /// </summary>
    public class OrderItemDTO
    {
        public int ID { get; set; }

        [Range(0, int.MaxValue)]
        public int Quantity { get; set; }

        [Required]
        public ProductDTO Product { get; set; }

        protected bool Equals(OrderItemDTO other)
        {
            if (ID > 0)
            {
                return this.ID == other.ID;
            }
            return Quantity == other.Quantity && 
                Equals(Product, other.Product);
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj))
            {
                return false;
            }
            if (ReferenceEquals(this, obj))
            {
                return true;
            }
            return obj.GetType() == this.GetType() &&
                Equals((OrderItemDTO) obj);
        }

        public override int GetHashCode()
        {
            unchecked
            {
                var hashCode = ID;
                hashCode = (hashCode*397) ^ Quantity;
                hashCode = (hashCode*397) ^ (Product?.GetHashCode() ?? 0);
                return hashCode;
            }
        }

        public override string ToString()
        {
            return $"{Product} {Quantity}x";
        }
    }
}
