﻿using System.ComponentModel.DataAnnotations;
using DAL.Enums;

namespace BL.DTOs.Products
{
    public class ProductDTO
    {
        public int ID { get; set; }

        [Required]
        [MaxLength(1024)]
        public string Name { get; set; }

        [MaxLength(65536)]
        [DataType(DataType.MultilineText)]
        public string Description { get; set; }

        [Range(0.0, 10.0)]
        public double? AverageRating { get; set; }

        [Range(0, int.MaxValue)]
        [DisplayFormat(DataFormatString = "{0:#.####}", ApplyFormatInEditMode = true)]
        public decimal Price { get; set; }

        public DiscountType DiscountType { get; set; }

        [Range(0, 100)]
        public int? DiscountPercentage { get; set; }

        [MaxLength(1024)]
        public string ProductImgUri { get; set; }

        public int CategoryID { get; set; }

        public int StockID { get; set; }

        public int? CurrentlyAvailableUnits { get; set; }

        public override string ToString()
        {
            return Name;
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj))
            {
                return false;
            }
            if (ReferenceEquals(this, obj))
            {
                return true;
            }
            return obj.GetType() == this.GetType() && ((ProductDTO)obj).GetHashCode() == this.GetHashCode();
        }

        public override int GetHashCode()
        {
            if (ID > 0)
            {
                return ID;
            }
            unchecked
            {
                var hashCode = Name?.GetHashCode() ?? 0;
                hashCode = (hashCode * 397) ^ (Description?.GetHashCode() ?? 0);
                hashCode = (hashCode * 397) ^ Price.GetHashCode();
                hashCode = (hashCode * 397) ^ (int)DiscountType;
                hashCode = (hashCode * 397) ^ DiscountPercentage.GetHashCode();
                hashCode = (hashCode * 397) ^ (ProductImgUri?.GetHashCode() ?? 0);
                hashCode = (hashCode * 397) ^ CategoryID.GetHashCode();
                hashCode = (hashCode * 397) ^ StockID.GetHashCode();
                return hashCode;
            }
        }
    }
}
