﻿using System.ComponentModel.DataAnnotations;
using BL.DTOs.Products;

namespace BL.DTOs.Reviews
{
    public class ProductReviewDTO
    {
        public int ID { get; set; }

        [Required]
        [Range(0.0, 10.0)]
        public double UserRating { get; set; }

        [Required]
        [StringLength(32, MinimumLength = 2)]
        public string Author { get; set; }

        [MaxLength(65536)]
        public string Review { get; set; }

        [MaxLength(16384)]
        public string Pros { get; set; }

        [MaxLength(16384)]
        public string Cons { get; set; }

        public int ProductID { get; set; }

        public override string ToString()
        {
            return $"Review for {ProductID} with rating: {UserRating}/10";
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj))
            {
                return false;
            }
            if (ReferenceEquals(this, obj))
            {
                return true;
            }
            return obj.GetType() == this.GetType() && ((ProductReviewDTO)obj).GetHashCode() == this.GetHashCode();
        }

        public override int GetHashCode()
        {
            if (ID > 0)
            {
                return ID;
            }
            unchecked
            {
                var hashCode = UserRating.GetHashCode();
                hashCode = (hashCode * 397) ^ (Author?.GetHashCode() ?? 0);
                hashCode = (hashCode * 397) ^ (Review?.GetHashCode() ?? 0);
                hashCode = (hashCode * 397) ^ (Pros?.GetHashCode() ?? 0);
                hashCode = (hashCode * 397) ^ (Cons?.GetHashCode() ?? 0);
                hashCode = (hashCode * 397) ^ ProductID.GetHashCode();
                return hashCode;
            }
        }
    }
}
