﻿using System;
using System.ComponentModel.DataAnnotations;

namespace BL.DTOs.Stocks
{
    /// <summary>
    /// Represents stock reservation made by customer
    /// </summary>
    public class StockReservationDTO
    {
        /// <summary>
        /// ID of the corresponding stock
        /// </summary>
        [Range(1, int.MaxValue)]
        public int StockId { get; set; }

        /// <summary>
        /// ID of customer who made the reservation
        /// </summary>
        [Range(1, int.MaxValue)]
        public int CustomerId { get; set; }

        /// <summary>
        /// Amount of units reserved
        /// </summary>
        [Range(0, int.MaxValue)]
        public int ReservedAmount { get; set; }

        /// <summary>
        /// Reservation expiration
        /// </summary>
        public DateTime Expiration { get; set; }

        public override string ToString()
        {
            return $"Customer with ID: {CustomerId} reserved {ReservedAmount} units of stock" +
                        $" with ID: {StockId} with expiration: {Expiration.ToShortTimeString()}";
        }

        protected bool Equals(StockReservationDTO other)
        {
            return StockId == other.StockId && CustomerId == other.CustomerId && 
                ReservedAmount == other.ReservedAmount && Expiration.Equals(other.Expiration);
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj))
            {
                return false;
            }
            if (ReferenceEquals(this, obj))
            {
                return true;
            }
            return obj.GetType() == this.GetType() && Equals((StockReservationDTO) obj);
        }

        public override int GetHashCode()
        {
            unchecked
            {
                var hashCode = StockId;
                hashCode = (hashCode*397) ^ CustomerId;
                hashCode = (hashCode*397) ^ ReservedAmount;
                hashCode = (hashCode * 397) ^ Expiration.GetHashCode();
                return hashCode;
            }
        }

    }
}
