﻿using System;
using System.Collections.Generic;
using BL.DTOs.Customers;
using BL.DTOs.Filters;
using BL.DTOs.Orders;
using BL.Services.Customers;
using BL.Services.Promote;

namespace BL.Facades
{
    public class CustomerFacade
    {
        public int CustomerPageSize => customerService.CustomersPageSize;

        #region Dependencies

        private readonly ICustomerService customerService;

        private readonly IPromoteService promoteService; 

        public CustomerFacade(ICustomerService customerService, IPromoteService promoteService)
        {
            this.customerService = customerService;
            this.promoteService = promoteService;
        }

        #endregion
        
        #region Customer/User management

        /// <summary>
        /// Gets customer (including its user account) according to email
        /// </summary>
        /// <param name="email"></param>
        /// <returns>Customer with specified email</returns>
        public CustomerDTO GetCustomerAccordingToEmail(string email)
        {
            return customerService.GetCustomerAccordingToEmail(email);
        }

        /// <summary>
        /// Gets all customers according to page
        /// </summary>
        /// <param name="requiredPage">page to display</param>
        /// <returns>all customers</returns>
        public CustomerListQueryResultDTO GetCustomers(int requiredPage = 1)
        {
            return customerService.ListCustomers(requiredPage);
        }

        /// <summary>
        /// Gets all orders according to filter and required page
        /// </summary>
        /// <param name="filter">order filter</param>
        /// <param name="requiredPage">page to show</param>
        /// <returns>All orders</returns>
        public IEnumerable<OrderDTO> GetOrders(OrderFilter filter, int requiredPage = 1)
        {
            return customerService.ListOrders(filter, requiredPage);
        }

        #endregion

        #region CustomersSpecialOffers

        /// <summary>
        /// Sends loyalty coupon if and only if all conditions are satisfied
        /// </summary>
        /// <param name="customerId">ID of coupon receiver</param>
        /// <returns>true if coupon was sent</returns>
        public bool SendLoyaltyCouponIfPossible(int customerId)
        {
            return promoteService.SendLoyaltyCouponIfPossible(customerId);
        }

        /// <summary>
        /// Sends birthday coupon if and only if all conditions are satisfied
        /// </summary>
        /// <param name="customerId">ID of coupon receiver</param>
        /// <returns>true if coupon was sent</returns>
        public bool SendBirthDayCouponIfPossible(int customerId)
        {
            return promoteService.SendBirthDayCouponIfPossible(customerId);
        } 

        #endregion
    }
}
