﻿using System.Linq;
using AutoMapper.QueryableExtensions;
using BL.AppInfrastructure;
using BL.DTOs.Coupons;
using BL.DTOs.Filters;
using DAL.Entities;
using Riganti.Utils.Infrastructure.Core;

namespace BL.Queries
{
    public class CouponListQuery : AppQuery<CouponDTO>
    {
        public CouponFilter Filter { get; set; }

        public CouponListQuery(IUnitOfWorkProvider provider) : base(provider) { }

        protected override IQueryable<CouponDTO> GetQueryable()
        {
            IQueryable<Coupon> query = Context.Coupons;

            if (Filter.CustomerId > 0)
            {
                query = query.Where(coupon => coupon.Customer.ID == Filter.CustomerId);
            }
            if (!string.IsNullOrEmpty(Filter.PromoCode))
            {
                query = query.Where(coupon => coupon.PromoCode.Equals(Filter.PromoCode));
            }
            if (Filter.OrderId > 0)
            {
                query = query.Where(coupon => coupon.Order != null && coupon.Order.ID == Filter.OrderId);
            }
            if (Filter.IsUsed != null)
            {
                query = query.Where(order => order.Order != null == Filter.IsUsed.Value);
            }

            return query.ProjectTo<CouponDTO>();
        }
    }
}
