﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using BL.AppInfrastructure;
using BL.DTOs.Customers;
using DAL.Entities;
using Riganti.Utils.Infrastructure.Core;

namespace BL.Queries
{
    public class CustomerAccordingToEmailQuery : AppQuery<CustomerDTO>
    {
        public CustomerAccordingToEmailQuery(IUnitOfWorkProvider provider) : base(provider) { }

        public string Email { get; set; }

        protected override IQueryable<CustomerDTO> GetQueryable()
        {
            if (string.IsNullOrEmpty(Email) || !new EmailAddressAttribute().IsValid(Email))
            {
                throw new InvalidOperationException("CustomerAccordingToUserIdQuery - Email must be valid.");
            }

            // Single result is expected so client side execution is not a problem
            var customer = Context.Customers.Include(nameof(Customer.Account))
                .FirstOrDefault(c => c.Account.Email.Equals(Email));

            if (customer == null)
            {
                return new EnumerableQuery<CustomerDTO>(new List<CustomerDTO>());
            }

            var customerDto = AutoMapper.Mapper.Map<CustomerDTO>(customer);

            return new EnumerableQuery<CustomerDTO>(new List<CustomerDTO> { customerDto });           
        }
    }
}
