﻿using System.Collections.Generic;
using System.Linq;
using AutoMapper.QueryableExtensions;
using BL.AppInfrastructure;
using BL.DTOs.Customers;
using Riganti.Utils.Infrastructure.Core;

namespace BL.Queries
{
    public class CustomerCountPerProductQuery : AppQuery<CustomerDTO>
    {
        public int ProductId { get; set; }

        public CustomerCountPerProductQuery(IUnitOfWorkProvider provider) : base(provider) { }

        protected override IQueryable<CustomerDTO> GetQueryable()
        {
            if (ProductId <= 0)
            {
                return new EnumerableQuery<CustomerDTO>(new List<CustomerDTO>());
            }
            return Context.Customers
                .Where(customer => customer.Orders
                    .Any(order => order.OrderItems
                        .Any(item => item.Product.ID == ProductId)))
                .ProjectTo<CustomerDTO>();
        }
    }
}
