﻿using System.Linq;
using AutoMapper.QueryableExtensions;
using BL.AppInfrastructure;
using BL.DTOs.Filters;
using BL.DTOs.Orders;
using DAL.Entities;
using Riganti.Utils.Infrastructure.Core;

namespace BL.Queries
{
    public class OrderListQuery : AppQuery<OrderDTO>
    {
        public OrderFilter Filter { get; set; }

        public OrderListQuery(IUnitOfWorkProvider provider) : base(provider) { }

        protected override IQueryable<OrderDTO> GetQueryable()
        {
            IQueryable<Order> query = Context.Orders;

            if (Filter.CustomerId > 0)
            {
                query = query.Where(order => order.Customer.ID == Filter.CustomerId);
            }
            if (Filter.ProductId > 0)
            {
                query = query.Where(order => order.OrderItems.Any(item => item.Product.ID == Filter.ProductId));
            }           
            if (Filter.IssuedRange != null)
            {
                query = query.Where(order => order.Issued >= Filter.IssuedRange.Minimum && order.Issued <= Filter.IssuedRange.Maximum);
            }

            return query.ProjectTo<OrderDTO>();
        }
    }
}
