﻿using System.Collections.Generic;
using System.Linq;
using AutoMapper.QueryableExtensions;
using BL.AppInfrastructure;
using BL.DTOs.Filters;
using BL.DTOs.Products;
using DAL.Entities;
using Riganti.Utils.Infrastructure.Core;

namespace BL.Queries
{
    public class ProductListQuery : AppQuery<ProductDTO>
    {
        public ProductFilter Filter { get; set; }

        public ProductListQuery(IUnitOfWorkProvider provider) : base(provider) { }

        protected override IQueryable<ProductDTO> GetQueryable()
        {
            IQueryable<Product> query = Context.Products
                                            .Include(nameof(Product.ProductReviews))
                                            .Include(nameof(Product.Stock));

            if (Filter.CategoryIds != null && Filter.CategoryIds.Any())
            {
                query = query.Where(product => Filter.CategoryIds.Contains(product.Category.ID) );
            }
            if (Filter.MinimalRating > 0)
            {
                query = query.Where(product => product.ProductReviews.Average(review => review.UserRating) >= Filter.MinimalRating);
            }
            if (!string.IsNullOrEmpty(Filter.ProductName))
            {
                query = query.Where(product => product.Name.ToLower().Contains(Filter.ProductName.ToLower()));
            }
            if (Filter.PriceRange != null && Filter.PriceRange.Maximum > 0)
            {
                query = query.Where(product => product.Price >= Filter.PriceRange.Minimum && product.Price <= Filter.PriceRange.Maximum);
            }
            return query.ProjectTo<ProductDTO>();
        }

        protected override void PostProcessResults(IList<ProductDTO> results)
        {
            foreach (var productDto in results)
            {
                if (Context.ProductReviews.Any())
                {
                    var rating = Context.ProductReviews
                        .Where(review => review.Product.ID == productDto.ID)
                        .Average(rev => (double?)(rev.UserRating));
                    productDto.AverageRating = rating;
                }
            }

            base.PostProcessResults(results);
        }
    }
}
