﻿using System.Collections.Generic;
using BL.DTOs.Categories;

namespace BL.Services.Categories
{
    /// <summary>
    /// Handles Category CRUD
    /// </summary>
    public interface ICategoryService
    {
        /// <summary>
        /// Creates new category
        /// </summary>
        /// <param name="categoryDto">category details</param>
        void CreateCategory(CategoryDTO categoryDto);

        /// <summary>
        /// Updates category according to ID
        /// </summary>
        /// <param name="categoryDto">category details</param>
        void EditCategory(CategoryDTO categoryDto);

        /// <summary>
        /// Removes category according to ID
        /// </summary>
        /// <param name="categoryId">category ID</param>
        void DeleteCategory(int categoryId);

        /// <summary>
        /// Gets category according to ID
        /// </summary>
        /// <param name="categoryId">category ID</param>
        /// <returns>The category</returns>
        CategoryDTO GetCategory(int categoryId);

        /// <summary>
        /// Gets category id that belongs to specified category name
        /// </summary>
        /// <param name="name">category name</param>
        /// <returns>id of category with specified name</returns>
        int GetProductCategoryIdByName(string name);

        /// <summary>
        /// Gets all parent categories for specified category
        /// </summary>
        /// <param name="categoryId">category id</param>
        /// <returns>all parent categories</returns>
        IEnumerable<CategoryDTO> GetCategoryPath(int categoryId);

        /// <summary>
        /// Gets all categories
        /// </summary>
        /// <returns>all available categories</returns>
        IEnumerable<CategoryDTO> ListAllCategories();
    }
}