using System;
using System.Collections.Generic;
using BL.DTOs.Coupons;
using BL.DTOs.Orders;

namespace BL.Services.Checkout
{
    /// <summary>
    /// Handles order checkout related functionality
    /// </summary>
    public interface ICheckoutService
    {        
        /// <summary>
        /// Persists order together with all related data
        /// </summary>
        /// <param name="createOrderDto">wrapper for order, orderItems, customer and coupon</param>
        void ConfirmOrder(OrderCreateDTO createOrderDto, Action confirmationAction);

        /// <summary>
        /// Calculates total price for all order items (overall coupon discount is included)
        /// </summary>
        /// <param name="orderItems">all order items</param>
        /// <param name="couponId">coupon id</param>
        /// <returns></returns>
        decimal CalculateTotalPrice(ICollection<OrderItemDTO> orderItems, int couponId = 0);

        /// <summary>
        /// Notifies warehouse about realised order and requests corresponding order shipment
        /// </summary>
        void RequestOrderDelivery();

        /// <summary>
        /// Gets total price only for order items
        /// </summary>
        /// <param name="orderItems">all order items</param>
        /// <returns>The total price for all items</returns>
        decimal GetItemsTotalPrice(params OrderItemDTO[] orderItems);

        /// <summary>
        /// Gets discount coupon according to promo code
        /// </summary>
        /// <param name="promoCode">Coupon promo code</param>
        /// <param name="customerId">ID of customer who owns the coupon</param>
        /// <returns>Respective discount coupon</returns>
        CouponDTO GetCoupon(string promoCode, int customerId);
    }
}