﻿using System;
using System.Collections.Generic;
using BL.DTOs.Coupons;
using BL.DTOs.Customers;
using BL.DTOs.Filters;
using BL.DTOs.Orders;

namespace BL.Services.Customers
{
    public interface ICustomerService
    {
        int CustomersPageSize { get; }

        /// <summary>
        /// Creates new customer (user account must be created first)
        /// </summary>
        /// <param name="userAccountId">Customer user account ID</param>
        void CreateCustomer(Guid userAccountId);

        /// <summary>
        /// Updates customer according to ID
        /// </summary>
        /// <param name="customerDto">customer details</param>
        void EditCustomer(CustomerDTO customerDto);

        /// <summary>
        /// Gets customer according to ID
        /// </summary>
        /// <param name="customerId">customer ID</param>
        /// <returns>The customer</returns>
        CustomerDTO GetCustomer(int customerId);

        /// <summary>
        /// Gets customers according to requested page
        /// </summary>
        /// <returns>Customers page</returns>
        CustomerListQueryResultDTO ListCustomers(int requiredPage);

        /// <summary>
        /// Gets all orders according to filter and required page
        /// </summary>
        /// <param name="filter">order filter</param>
        /// <param name="requiredPage">page to show</param>
        /// <returns>All orders</returns>
        IEnumerable<OrderDTO> ListOrders(OrderFilter filter, int requiredPage = 1);

        /// <summary>
        /// Gets all coupons according to filter and required page
        /// </summary>
        /// <param name="filter">coupon filter</param>
        /// <param name="requiredPage">page to show</param>
        /// <returns>All coupons</returns>
        IEnumerable<CouponDTO> ListCoupons(CouponFilter filter, int requiredPage = 1);

        /// <summary>
        /// Gets customer with given email address
        /// </summary>
        /// <param name="email">email</param>
        /// <returns>Customer with given email address</returns>
        CustomerDTO GetCustomerAccordingToEmail(string email);
    }
}
