﻿using System;
using System.Collections.Generic;
using BL.DTOs.Filters;
using BL.DTOs.Orders;
using BL.Queries;
using Riganti.Utils.Infrastructure.Core;

namespace BL.Services.Sales
{
    /// <summary>
    /// Provides statistics concerning e-shop sales
    /// </summary>
    public class SalesService : EshopService, ISalesService
    {
        public int OrderItemsPageSize = 10;

        #region Dependencies

        private readonly OrderItemListQuery orderItemListQuery;

        public SalesService(OrderItemListQuery orderItemListQuery)
        {
            this.orderItemListQuery = orderItemListQuery;
        } 

        #endregion

        /// <summary>
        /// Gets all OrderItems from specified order
        /// </summary>
        /// <param name="filter">order item filter</param>
        /// <param name="requiredPage">page to show</param>
        /// <returns>all order items from specified order</returns>
        public IEnumerable<OrderItemDTO> ListItemsFromOrder(OrderItemFilter filter, int requiredPage = 1)
        {
            using (UnitOfWorkProvider.Create())
            {
                var query = GetOrderItemQuery(filter);
                query.Skip = Math.Max(0, requiredPage - 1) * OrderItemsPageSize;
                query.Take = OrderItemsPageSize;

                var sortOrder = filter.SortAscending ? SortDirection.Ascending : SortDirection.Descending;
                query.AddSortCriteria(nameof(OrderItemDTO.ID), sortOrder);

                return query.Execute();
            }
        }

        /// <summary>
        /// Configures order item list query
        /// </summary>
        /// <param name="filter">order item filter</param>
        /// <returns>configured query</returns>
        private IQuery<OrderItemDTO> GetOrderItemQuery(OrderItemFilter filter)
        {
            var query = orderItemListQuery;
            query.ClearSortCriterias();
            query.Filter = filter;
            return query;
        }
    }
}
