using BL.DTOs.Stocks;

namespace BL.Services.Stocks
{
    public interface IStockService
    {
        /// <summary>
        /// Performs stock reservation in order to ensure
        /// simultaneous orders can be satisfied
        /// </summary>
        /// <param name="stockReservation">Stock reservation details containing 
        /// customer reserving the stock, amount, stockID and expiration</param>
        /// <returns>true if reservation is succesfull, otherwise false</returns>
        bool ReserveStock(StockReservationDTO stockReservation);

        /// <summary>
        /// Performs stock release 
        /// (typically when user confirms or cancels the order)
        /// </summary>
        /// <param name="customerId">ID of customer who reserved the stock</param>
        /// <param name="stockIds">IDs of the stock to release</param>
        void ReleaseStockReservations(int customerId, params int[] stockIds);

        /// <summary>
        /// Performs stock expedition by decreasing stocks 
        /// stored units and releasing corresponding reservations
        /// </summary>
        /// <param name="customerId">Id of customer whose reserved stock should be dispatched</param>
        void DispatchStock(int customerId);

        /// <summary>
        /// Performs order of required stock from distributor
        /// </summary>
        /// <param name="stockId">id of stock to order</param>
        /// <param name="quantity">Number of units to order</param>
        void OrderStockFromDistributor(int stockId, int quantity);

        /// <summary>
        /// Gets number of currently available (free)
        /// units for stock belonging to product with specified ID
        /// </summary>
        /// <param name="stockId">ID of the stock for which stock availability should be checked</param>
        /// <returns>Number of free units for this product</returns>
        int GetCurrentlyAvailableUnits(int stockId);
    }
}