﻿using System;
using System.ComponentModel.DataAnnotations;

namespace BL.Utils.Validation
{
    [AttributeUsage(AttributeTargets.Property)]
    public class RangeValidator : ValidationAttribute
    {
        protected override ValidationResult IsValid(object value, ValidationContext validationContext)
        {
            var minimum = GetPropertyValue(value, "Minimum", validationContext);
            var maximum = GetPropertyValue(value, "Maximum", validationContext);

            if (minimum == null || maximum == null)
            {
                return new ValidationResult("Instance must contain properties Minimum and Maximum in order to perform Range validation");
            }

            var result = minimum.CompareTo(maximum);

            if (result > 0) // min > max
            {
                return new ValidationResult("Minimum must not be greater than Maximum");
            }
            return ValidationResult.Success;            
        }

        /// <summary>
        /// It is not possible to create generic validation attribute 
        /// directly, therefore reflection must be used
        /// </summary>
        /// <param name="value">instance to get property value from</param>
        /// <param name="propName"></param>
        /// <returns></returns>
        private static IComparable GetPropertyValue(object value, string propName, ValidationContext validationContext)
        {
            var prop = validationContext.ObjectType.GetProperty(propName);
            return prop?.GetValue(value) as IComparable;
        }
    }
}

