﻿using System.Data.Entity;
using DAL.Entities;
using Riganti.Utils.Infrastructure.EntityFramework;

namespace DAL
{
    public class DemoEshopDbContext : DbContext
    {
        #region Ctors

        public DemoEshopDbContext() : base("DemoEshopDBContext")
        {
            InitializeDbContext();
        }

        public DemoEshopDbContext(string connectionName) : base(connectionName)
        {
            InitializeDbContext();
        } 

        #endregion

        public DbSet<Category> Categories { get; set; }
        public DbSet<Coupon> Coupons { get; set; }
        public DbSet<Customer> Customers { get; set; }
        public DbSet<Order> Orders { get; set; }
        public DbSet<OrderItem> OrderItems { get; set; }
        public DbSet<Product> Products { get; set; }
        public DbSet<ProductReview> ProductReviews { get; set; }
        public DbSet<UserAccount> Users { get; set; }
        public DbSet<Stock> Stocks { get; set; }

        protected override void OnModelCreating(DbModelBuilder modelBuilder)
        {
            modelBuilder.ConfigureMembershipRebootUserAccounts<UserAccount>();

            modelBuilder.Entity<Order>()
                .HasOptional(order => order.Coupon)
                .WithOptionalDependent(coupon => coupon.Order);

            modelBuilder.Entity<Product>()
                .HasRequired(product => product.Stock)
                .WithRequiredDependent(stock => stock.Product);
        }

        private void InitializeDbContext()
        {
            Database.SetInitializer(new DemoEshopDbInitializer());
            this.RegisterUserAccountChildTablesForDelete<UserAccount>();
        }

    }
}
