﻿using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using DAL.Enums;
using Riganti.Utils.Infrastructure.Core;

namespace DAL.Entities
{
    /// <summary>
    /// Eshop featured product
    /// </summary>
    public class Product : IEntity<int>
    {
        public int ID { get; set; }

        [Required]
        [MaxLength(1024)]
        public string Name { get; set; }

        [MaxLength(65536)]
        public string Description { get; set; }
      
        [Range(0, int.MaxValue)]
        public decimal Price { get; set; }

        public DiscountType DiscountType { get; set; }

        [Range(0, 100)]
        public int? DiscountPercentage { get; set; }

        [MaxLength(1024)]
        public string ProductImgUri { get; set; }

        [Required]
        public virtual Category Category { get; set; }

        [Required]
        public virtual Stock Stock { get; set; }

        public virtual List<ProductReview> ProductReviews { get; set; }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj))
            {
                return false;
            }
            if (ReferenceEquals(this, obj))
            {
                return true;
            }
            return obj.GetType() == this.GetType() && ((Product)obj).GetHashCode() == this.GetHashCode();
        }

        public override int GetHashCode()
        {
            if (ID > 0)
            {
                return ID;
            }
            unchecked
            {
                var hashCode = Name?.GetHashCode() ?? 0;
                hashCode = (hashCode*397) ^ (Description?.GetHashCode() ?? 0);
                hashCode = (hashCode*397) ^ Price.GetHashCode();
                hashCode = (hashCode*397) ^ (int) DiscountType;
                hashCode = (hashCode*397) ^ DiscountPercentage.GetHashCode();
                hashCode = (hashCode*397) ^ (ProductImgUri?.GetHashCode() ?? 0);
                hashCode = (hashCode*397) ^ (Category?.GetHashCode() ?? 0);
                hashCode = (hashCode * 397) ^ (Stock?.GetHashCode() ?? 0);
                return hashCode;
            }
        }
    }
}
