﻿using System.ComponentModel.DataAnnotations;
using Riganti.Utils.Infrastructure.Core;

namespace DAL.Entities
{
    /// <summary>
    /// Anonymous user review for product
    /// </summary>
    public class ProductReview : IEntity<int>
    {
        public int ID { get; set; }

        [Required]
        [Range(0.0, 10.0)]
        public double UserRating { get; set; }

        [Required]
        [StringLength(32, MinimumLength = 2)]
        public string Author { get; set; }

        [MaxLength(65536)]
        [DataType(DataType.MultilineText)]
        public string Review { get; set; }

        [MaxLength(16384)]
        public string Pros { get; set; }

        [MaxLength(16384)]
        public string Cons { get; set; }

        [Required]
        public virtual Product Product { get; set; }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj))
            {
                return false;
            }
            if (ReferenceEquals(this, obj))
            {
                return true;
            }
            return obj.GetType() == this.GetType() && ((ProductReview) obj).GetHashCode() == this.GetHashCode();
        }

        public override int GetHashCode()
        {
            if (ID > 0)
            {
                return ID;
            }
            unchecked
            {
                var hashCode = UserRating.GetHashCode();
                hashCode = (hashCode * 397) ^ (Author?.GetHashCode() ?? 0);
                hashCode = (hashCode*397) ^ (Review?.GetHashCode() ?? 0);
                hashCode = (hashCode*397) ^ (Pros?.GetHashCode() ?? 0);
                hashCode = (hashCode*397) ^ (Cons?.GetHashCode() ?? 0);
                hashCode = (hashCode*397) ^ (Product?.GetHashCode() ?? 0);
                return hashCode;
            }
        }
    }
}
