﻿using System.Web.Mvc;
using BL.DTOs.Filters;
using BL.Facades;
using BL.Services.Categories;
using BL.Services.Products;
using BL.Services.Rating;
using BL.Utils.AccountPolicy;
using PL.Models;
using System.Linq;
using BL.DTOs.Products;
using BL.Services.Stocks;
using X.PagedList;

namespace PL.Controllers.Admin
{
    public class AdminProductsController : Controller
    {
        public ProductFacade ProductFacade { get; set; }
        public OrderFacade OrderFacade { get; set; }

        // GET: AdminProducts
        public ActionResult Index(int page = 1)
        {
            var result = ProductFacade.GetProducts(new ProductFilter(), page);

            var model = new StaticPagedList<ProductDTO>(result.ResultsPage, page, ProductFacade.ProductPageSize,
                result.TotalResultCount);

            return View(model);
        }

        // GET: AdminProducts/Details/5
        public ActionResult Details(int id)
        {
            var model = ProductFacade.GetProductDetails(id);
            
            return View(model);
        }

        public ActionResult DeleteReview(int id, int productId)
        {
            ProductFacade.DeleteReview(id);
            return RedirectToAction("Details", new {id = productId});
        }

        // GET: AdminProducts/Create
        public ActionResult Create()
        {
            return View();
        }

        // POST: AdminProducts/Create
        [HttpPost]
        public ActionResult Create(AdminProductEditModel model)
        {
            try
            {
                ProductFacade.CreateProductWithCategoryName(model.Product, model.Category.Name);

                return RedirectToAction("Index");
            }
            catch
            {
                return View();
            }
        }

        // GET: AdminProducts/Edit/5
        public ActionResult Edit(int id)
        {
            var product = ProductFacade.GetProduct(id);
            var category = ProductFacade.GetCategory(product.CategoryID);

            var model = new AdminProductEditModel()
            {
                Product = product,
                Category = category
            };
            return View(model);
        }

        // POST: AdminProducts/Edit/5
        [HttpPost]
        public ActionResult Edit(int id, AdminProductEditModel model)
        {
            try
            {
                var categoryId = ProductFacade.GetProductCategoryIdByName(model.Category.Name);
                
                var reviews = ProductFacade.GetAllReviews(model.Product.ID);

                var reviewIds = reviews.Select(a => a.ID).ToArray();
                
                ProductFacade.EditProduct(model.Product, categoryId, reviewIds);
                return RedirectToAction("Index");
            }
            catch
            {
                return View();
            }
        }

        // GET: AdminProducts/Delete/5
        public ActionResult Delete(int id)
        {
            ProductFacade.DeleteProduct(id);

            return RedirectToAction("Index");
        }

        [HttpPost]
        public ActionResult OrderStock(int id, ProductDetailDTO model)
        {
            OrderFacade.OrderStockFromDistributor(id, model.OrderedProductNumber);

            return RedirectToAction("Details", new { id });
        }
    }
}
